"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomMapperFetcher = void 0;
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
class CustomMapperFetcher {
    constructor(visitor, customFetcher) {
        this.visitor = visitor;
        if (typeof customFetcher === 'undefined') {
            customFetcher = {
                queryFunc: 'createQueryHook',
                queryAndQueryClientFunc: 'createQueryAndQueryClientHook',
                infiniteQueryFunc: 'createInfiniteQueryHook',
                mutationFunc: 'createMutationHook',
                isQueryAndQueryClient: true,
            };
        }
        this._mapperQuery = (0, visitor_plugin_common_1.parseMapper)(customFetcher.queryFunc);
        this._mapperQueryClient = (0, visitor_plugin_common_1.parseMapper)(customFetcher.queryAndQueryClientFunc);
        this._mapperInfiniteQuery = (0, visitor_plugin_common_1.parseMapper)(customFetcher.infiniteQueryFunc);
        this._mapperMutation = (0, visitor_plugin_common_1.parseMapper)(customFetcher.mutationFunc);
        this._isQueryAndQueryClient = customFetcher.isQueryAndQueryClient;
    }
    getFetcherQueryFnName() {
        return this._mapperQuery.type;
    }
    getFetcherInfiniteQueryFnName() {
        return this._mapperInfiniteQuery.type;
    }
    getFetcherQueryClientFnName() {
        return this._mapperQueryClient.type;
    }
    getFetcherMutationFnName() {
        return this._mapperMutation.type;
    }
    getSubscriptionFnName(operationResultType, operationVariablesTypes) {
        return `useSubscription<TData, ${operationVariablesTypes}>`;
    }
    generateFetcherImplementaion() {
        if (this._mapperQuery.isExternal) {
            return (0, visitor_plugin_common_1.buildMapperImport)(this._mapperQuery.source, [
                {
                    identifier: this._mapperQuery.type,
                    asDefault: this._mapperQuery.default,
                },
                {
                    identifier: this._mapperQueryClient.type,
                    asDefault: this._mapperQueryClient.isExternal ? this._mapperQueryClient.default : false,
                },
                {
                    identifier: this._mapperInfiniteQuery.type,
                    asDefault: this._mapperInfiniteQuery.isExternal ? this._mapperInfiniteQuery.default : false,
                },
                {
                    identifier: this._mapperMutation.type,
                    asDefault: this._mapperMutation.isExternal ? this._mapperMutation.default : false,
                },
            ], this.visitor.config.useTypeImports);
        }
        return null;
    }
    generateInfiniteQueryHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables) {
        const hookConfig = this.visitor.queryMethodMap;
        this.visitor.reactQueryHookIdentifiersInUse.add(hookConfig.infiniteQuery.hook);
        this.visitor.reactQueryOptionsIdentifiersInUse.add(hookConfig.infiniteQuery.options);
        const typedFetcher = this.getFetcherInfiniteQueryFnName();
        const impl = `use${operationName} = ${typedFetcher}`;
        return `export const ${impl}<
   ${operationResultType}, 
   ${operationVariablesTypes}
>(${documentVariableName}, EQueryKey.${operationName});`;
    }
    generateQueryHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables) {
        const hookConfig = this.visitor.queryMethodMap;
        this.visitor.reactQueryHookIdentifiersInUse.add(hookConfig.query.hook);
        this.visitor.reactQueryOptionsIdentifiersInUse.add(hookConfig.query.options);
        const typedFetcher = this.getFetcherQueryFnName();
        const typedFetcherAndClient = this.getFetcherQueryClientFnName();
        const impl = this._isQueryAndQueryClient && !!typedFetcherAndClient
            ? `{useQuery: use${operationName}, useQueryClient: use${operationName}Client} = ${typedFetcherAndClient}`
            : `use${operationName} = ${typedFetcher}`;
        return `export const ${impl}<
   ${operationResultType}, 
   ${operationVariablesTypes}
>(${documentVariableName}, EQueryKey.${operationName});`;
    }
    generateMutationHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables) {
        const hookConfig = this.visitor.queryMethodMap;
        this.visitor.reactQueryHookIdentifiersInUse.add(hookConfig.mutation.hook);
        this.visitor.reactQueryOptionsIdentifiersInUse.add(hookConfig.mutation.options);
        const typedFetcher = this.getFetcherMutationFnName();
        return `export const use${operationName} = ${typedFetcher}<
    ${operationResultType}, 
    ${operationVariablesTypes}
>(${documentVariableName}, EMutationKey.${operationName});

`;
    }
    generateSubscriptionHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables) {
        const variables = `args${hasRequiredVariables ? '' : '?'}: SubscriptionHookOptions<TData, ${operationVariablesTypes}>`;
        const hookConfig = this.visitor.queryMethodMap;
        this.visitor.reactQueryHookIdentifiersInUse.add(hookConfig.query.hook);
        this.visitor.reactQueryOptionsIdentifiersInUse.add(hookConfig.query.options);
        const typedFetcher = this.getSubscriptionFnName(operationResultType, operationVariablesTypes);
        const impl = `${typedFetcher}(gql(${documentVariableName}), args);`;
        return `export const use${operationName} = <
      TData = ${operationResultType},
      TError = ${this.visitor.config.errorType}
    >(
      ${variables},
    ) =>
      ${impl}
    `;
    }
}
exports.CustomMapperFetcher = CustomMapperFetcher;
//# sourceMappingURL=fetcher-custom-mapper.js.map